import os
import unittest
from shutil import rmtree


class TestInterfaceGenerator(unittest.TestCase):

    def test_InterfaceGenerator(self):
        import AutoRPE.UtilsRPE.BasicStructures as BasicStructures
        from AutoRPE.Scripts.InterfaceGenerator import generate_routine_versions
        import itertools
        sbr = BasicStructures.SubRoutine("dia_hth_dep", None)
        var_1 = BasicStructures.Variable(sbr, "Kmm", _type="integer", _dimension=1, _module=None)
        var_2 = BasicStructures.Variable(sbr, "ptem", _type="double", _dimension=1, _module=None)
        var_3 = BasicStructures.Variable(sbr, "pdept", _type="double", _dimension=2, _module=None)
        var_1.is_dummy_argument = True
        var_2.is_dummy_argument = True
        var_3.is_dummy_argument = True
        var_4 = BasicStructures.Variable(sbr, "ii", _type="integer", _dimension=1, _module=None)
        var_5 = BasicStructures.Variable(sbr, "jj", _type="double", _dimension=1, _module=None)
        var_6 = BasicStructures.Variable(sbr, "zztmp", _type="double", _dimension=1, _module=None)
        var_7 = BasicStructures.Variable(sbr, "iktem", _type="integer", _dimension=2, _module=None)
        sbr.variables = [var_1, var_2, var_3, var_4, var_5, var_6, var_7]
        sensitive_variables = [v for v in sbr.variables if v.is_dummy_argument and v.type == "double"]
        precision_combinations = list(itertools.product(["dp", "sp"], repeat=len(sensitive_variables)))
        subprogram_code = ['SUBROUTINE dia_hth_dep( Kmm, ptem, pdept )\n',
                           '!\n',
                           '      INTEGER , INTENT(in) :: Kmm     '
                           '      ! ocean time level index\n',
                           '      REAL(wp), INTENT(in) :: ptem\n',
                           '      REAL(wp), DIMENSION(jpi,jpj), INTENT(out)  :: pdept     \n',
                           '!\n',
                           '      INTEGER  :: ii, jj\n',
                           '      REAL(wp) :: zztmp\n',
                           '      INTEGER, DIMENSION(jpi,jpj) :: iktem\n',
                           '\n',
                           '! --------------------------------------- !\n',
                           '! search deepest level above ptem         !\n',
                           '! --------------------------------------- !\n',
                           'END SUBROUTINE dia_hth_dep\n']
        expected_output = ['SUBROUTINE dia_hth_dep_0000( Kmm, ptem, pdept )\n', '!\n',
                           '      INTEGER , INTENT(in) :: Kmm           ! ocean time level index\n',
                           '      REAL(dp), INTENT(in) :: ptem\n',
                           '      REAL(dp), DIMENSION(jpi,jpj), INTENT(out)  :: pdept     \n', '!\n',
                           '      INTEGER  :: ii, jj\n', '      REAL(wp) :: zztmp\n',
                           '      INTEGER, DIMENSION(jpi,jpj) :: iktem\n', '\n',
                           '! --------------------------------------- !\n',
                           '! search deepest level above ptem         !\n',
                           '! --------------------------------------- !\n',
                           'END SUBROUTINE dia_hth_dep_0000\n',
                           '\n',
                           '\n',
                           'SUBROUTINE dia_hth_dep_0001( Kmm, ptem, pdept )\n', '!\n',
                           '      INTEGER , INTENT(in) :: Kmm           ! ocean time level index\n',
                           '      REAL(dp), INTENT(in) :: ptem\n',
                           '      REAL(sp), DIMENSION(jpi,jpj), INTENT(out)  :: pdept     \n', '!\n',
                           '      INTEGER  :: ii, jj\n', '      REAL(wp) :: zztmp\n',
                           '      INTEGER, DIMENSION(jpi,jpj) :: iktem\n', '\n',
                           '! --------------------------------------- !\n',
                           '! search deepest level above ptem         !\n',
                           '! --------------------------------------- !\n',
                           'END SUBROUTINE dia_hth_dep_0001\n',
                           '\n',
                           '\n',
                           'SUBROUTINE dia_hth_dep_0002( Kmm, ptem, pdept )\n', '!\n',
                           '      INTEGER , INTENT(in) :: Kmm           ! ocean time level index\n',
                           '      REAL(sp), INTENT(in) :: ptem\n',
                           '      REAL(dp), DIMENSION(jpi,jpj), INTENT(out)  :: pdept     \n', '!\n',
                           '      INTEGER  :: ii, jj\n', '      REAL(wp) :: zztmp\n',
                           '      INTEGER, DIMENSION(jpi,jpj) :: iktem\n', '\n',
                           '! --------------------------------------- !\n',
                           '! search deepest level above ptem         !\n',
                           '! --------------------------------------- !\n',
                           'END SUBROUTINE dia_hth_dep_0002\n',
                           '\n',
                           '\n',
                           'SUBROUTINE dia_hth_dep_0003( Kmm, ptem, pdept )\n', '!\n',
                           '      INTEGER , INTENT(in) :: Kmm           ! ocean time level index\n',
                           '      REAL(sp), INTENT(in) :: ptem\n',
                           '      REAL(sp), DIMENSION(jpi,jpj), INTENT(out)  :: pdept     \n', '!\n',
                           '      INTEGER  :: ii, jj\n', '      REAL(wp) :: zztmp\n',
                           '      INTEGER, DIMENSION(jpi,jpj) :: iktem\n', '\n',
                           '! --------------------------------------- !\n',
                           '! search deepest level above ptem         !\n',
                           '! --------------------------------------- !\n',
                           'END SUBROUTINE dia_hth_dep_0003\n',
                           '\n',
                           '\n']
        generate_routine_versions(precision_combinations, sensitive_variables, sbr, subprogram_code, "./")

        with open("./" + sbr.name + "_interfaces.f90", 'r') as f:
            output_lines = [l for l in f]
        os.remove("./" + sbr.name + "_interfaces.f90")
        self.assertEqual(expected_output, output_lines)


if __name__ == '__main__':
    unittest.main()
