import AutoRPE.UtilsRPE.BasicFunctions as BasicFunctions
from AutoRPE.UtilsRPE.Splitter import split_semicolon
import AutoRPE.UtilsRPE.CallManager as CallManager
import re


# Remove comments, empty lines and old style declaration
def pre_process_sources(path_to_input_sources, path_to_output_sources):
    all_sources = BasicFunctions.load_sources(path_to_input_sources)
    n_of_file = len(all_sources)
    for index, file in enumerate(all_sources):
        print("\rFormatting file %i/%i  %20s" % (index + 1, n_of_file, file.filename), end="")
        pre_process(file)
        file.save_file(path_to_output_sources)


def pre_process(file):
    # Remove all comments from all the lines
    remove_all_comments(file)
    # Deletes all the empty lines
    file.lines = BasicFunctions.remove_empty_lines(file.lines)
    # Merges commands split on multiples lines
    file.lines = BasicFunctions.merge_lines(file.lines)
    # Split multiple command lines
    split_lines(file)
    # Split lines with more than one declaration
    split_declarations(file)

    # # Just needed for old Nemo version
    # if replace_wrk_alloc:
    #     replace_workalloc_in_file(file)

    # Remove parameter assignation F77 style
    _remove_F77_parameter_declaration(file)
    # Substitutes all multiple spaces with single spaces
    remove_multiple_space(file)
    # Put text back together
    file.rebuild_text()


def split_lines(file):
    # Split lines which contains more than one command
    # TODO: We should be sure that we are not splitting by characters in literals
    _index = 0
    length = len(file.lines)
    while _index < length:
        line = file.lines[_index]
        if line.count(";"):
            parts = split_semicolon(line)
            file.lines[_index] = parts[0]
            for i in range(1, len(parts)):
                file.lines.insert(_index + 1, parts[i])
                length += 1
                _index += 1
        _index += 1
    file.rebuild_text()


def remove_multiple_space(file):
    import itertools
    for index, line in enumerate(file.lines):
        match_index = [[i.start(), i.end()] for i in re.finditer('\\s\\s+', line)][::-1]
        if match_index:
            literal_masks = BasicFunctions.literal_mask(line)
            line = list(itertools.chain.from_iterable([list(piece) for piece in list(line)]))
            for mi in match_index:
                if not literal_masks[mi[0]]:
                    line[mi[0]:mi[1]] = " "
            file.lines[index] = "".join(line)


def split_declarations(file):
    import AutoRPE.UtilsRPE.RegexPattern as RegexPattern
    # Split declarations of variables putting a single variable in each line
    _index = 0
    length = len(file.lines)
    while _index < length:
        line = file.lines[_index]
        line = BasicFunctions.remove_literals(line)
        if line.count("::") and not re.search(RegexPattern.string_array_initialization, line):
            attributes, arguments = line.split("::")
            arguments = CallManager.find_call_arguments("c(%s)" % arguments)
            if len(arguments) > 1:
                new_line = "%s :: %s" % (attributes, arguments[0])
                file.lines[_index] = new_line
                for argument in arguments[1:]:
                    new_line = "%s :: %s" % (attributes, argument)
                    file.lines.insert(_index + 1, new_line)
                    length += 1
                    _index += 1

        _index += 1
    file.rebuild_text()


def remove_all_comments(file):
    # Remove comments from a file
    for _index, _line in enumerate(file.lines):
        file.lines[_index] = BasicFunctions.remove_comments(_line)


def _remove_F77_parameter_declaration(file):
    # If parameter declaration is made in Fortran 77 style
    pattern = '^\s*PARAMETER\s*\(\s*[\w+=\s*]'

    for line_index, line in enumerate(file.lines):

        # Search for lines with the 'PARAMETER' statement
        match = re.match(pattern, line, re.I)
        if match:
            param_line = line.replace(")", "")

            # Store parameter names
            parameters_name = [parameter.split("=")[0].strip() for parameter in
                               param_line.split("(")[1].split(",")]

            # Store parameter values
            parameters_val = [parameter.split("=")[1].strip() for parameter in
                              param_line.split("(")[1].split(",")]

            for parameter_name, parameter_value in zip(parameters_name, parameters_val):

                # Search for each parameter declaration backward in the file
                for dec_line_index, dec_line in enumerate(reversed((file.lines[:line_index]))):
                    parameter_pattern = '.*::\s*' + parameter_name + '$'
                    param_declaration = re.match(parameter_pattern, dec_line, re.I)
                    if (line_index - dec_line_index - 1) == 0:
                        raise AssertionError("declaration of parameter %s not found in file %s" % (
                            parameter_name, file.filename))
                    # Put back in place the parameter attribute and assign the parameter vale
                    if param_declaration:
                        # Add param statement
                        piece_to_replace = dec_line.split('::')[0]
                        replacement = "%s, parameter " % piece_to_replace
                        dec_line = dec_line.replace(piece_to_replace, replacement)

                        # Add param assignation
                        dec_line = dec_line + "=" + parameter_value

                        # Substitute the line
                        file.lines[line_index - dec_line_index - 1] = dec_line

                        break

            # Delete parameter declaration
            file.lines[line_index] = ""
